from pathlib import Path
import os

def mkpackage(path: Path | str):
    try: os.mkdir(path) 
    except: ...
    with open(path / "__init__.py", mode="w") as fh:
        fh.write("")

def mkmodule(path: Path | str, text: str):
    with open(path, mode="w", encoding="utf-8") as fh:
        fh.write(text)


from .common import *

def create_screen(cwd: Path): 
    mkpackage(cwd)
    mkmodule(cwd / "welcome.py", """\
from .common import *

@botm.dynamic_screen("welcome")
async def menus(user_id: int, **kwargs):
    return [SimpleMessage("Меню", 
        ButtonRows(
             ButtonRow(Button("Пусто", Dummy()))
        ))]
""")
    mkmodule(cwd / "common.py", """\
from typing import Callable
from telegram import Message as TgMessage
from tg_bot_screen.ptb import SimpleMessage, PhotoMessage, ButtonRows, \
    ButtonRow, Button, GoToScreen, RunFunc, StepBack, ScreenCallback, \
    FuncCallback, InputSession, FuncCallback, Message, DocumentMessage, \
    VideoMessage
from tg_bot_screen.callback_data import Dummy
from tg_bot_screen.ptb.user_data import UserData as SysUserData
from src.init.app import botm

def step_back_button_row(text: str = "↖️ Вернуться назад", 
        times: int = 1):
    return ButtonRow(Button(text, StepBack(times)))

def mdbq(text: str):
    "Добавить blockquote к тексту"
    return f"<blockquote>{text}</blockquote>"

def mdb(text: str):
    "Добавить bold к тексту"
    return f"<b>{text}</b>"

def mdi(text: str):
    "Добавить italic к тексту"
    return f"<i>{text}</i>"

async def is_bad_text(user_id: int, message: TgMessage = None, stack = True):
    if message.text is None:
        await botm.screen.set_by_name(user_id, "bad_text", stack=stack)
        return True
    return False

@botm.dynamic_screen("bad_text")
async def menus(user_id: int, message: TgMessage = None, **kwargs):
    return [SimpleMessage("😔 Неподходящий текст или он отсутствует",
            ButtonRows(step_back_button_row()))]

async def is_bad_image(user_id: int, message: TgMessage = None, stack = True):
    if message.photo is None:
        await botm.screen.set_by_name(user_id, "bad_image", stack=stack)
        return True
    return False

@botm.dynamic_screen("bad_image")
async def menus(user_id: int, message: TgMessage = None, **kwargs):
    return [SimpleMessage("😔 Неподходящая картинка или она отсутствует",
            ButtonRows(step_back_button_row()))]

def simple_screen(text: str, has_step_back = True, parse_mode: str = None,
        step_back_times: int = 1):
    button_rows = ButtonRows()
    if has_step_back:
        button_rows.append(step_back_button_row(times=step_back_times))
    return [SimpleMessage(text, button_rows, parse_mode)]

def select_screen(text: str, yes_func: Callable, parse_mode: str = None):
    return [SimpleMessage(text, 
        ButtonRows(
             ButtonRow(Button("Да", RunFunc(yes_func)))
            ,step_back_button_row())
        , parse_mode)]

def simple_multi_screen(text: str, has_step_back = True, parse_mode: str = None,
        step_back_times: int = 1):
    button_rows = ButtonRows()
    if has_step_back:
        button_rows.append(step_back_button_row(times=step_back_times))
    texts: list[str] = []
    cap = 4096
    while text != "":
        texts.append(text[:cap])
        text = text[cap:]
    msgs = [SimpleMessage(text, parse_mode=parse_mode) for text in texts]
    msgs[-1].button_rows = button_rows
    return msgs

@botm.dynamic_screen("error_screen")
async def menus(user_id: int, **kwargs):
    user_data = botm.get_user_data(user_id)
    return [SimpleMessage(f"😔 Ошибка: {user_data.last_error}",
            ButtonRows(step_back_button_row()))]""")


import os
import argparse
from pathlib import Path
from .project_generator.common import *
from .project_generator.create_init import create_init
from .project_generator.create_model import create_model
from .project_generator.create_screen import create_screen
from .project_generator.create_types import create_types


def main():
    parser = argparse.ArgumentParser(
        description='python -m tg_bot_screen', 
        usage='%(prog)s [options]')
    parser.add_argument("--ptb", "--python-telegram-bot", action="store_true")
    
    args = parser.parse_args()
    
    if args.ptb is False:
        good_use = R"python .\tg_bot_screen\__main__.py --ptb"
        print(f"""
Ошибка: Не передан аргумент, который отражает библиотеку, которая будет использована
Например --ptb
\tИспользование:
\t{good_use}
""")
        return
    
    mkmodule("run.py", "from src.init import main")
    basewd = Path("src") # Working Directory
    
    mkpackage(basewd)
    
    create_init(basewd / "init")
    
    create_model(basewd / "model")
    
    create_screen(basewd / "screen")
    
    create_types(basewd / "types")


if __name__ == "__main__":
    main()



from .common import *

def create_init(cwd: Path): 
    mkpackage(cwd)
    mkmodule(cwd / "app.py", """\
from os import environ
from telegram.ext import Application
from src.model.bot_manager import BotManager

token = environ.get("BOT_TOKEN")

application = Application.builder().token(token).build()

bot = application.bot

botm = BotManager(application).build()""")
    
    mkmodule(cwd / "config.py", """\
from pathlib import Path
from json import loads, dumps
from .app import botm

file_path = Path("config/config.json")
file_path.parent.mkdir(parents=True, exist_ok=True)
defaults = botm.config.defaults
    
def load_config():
    check_fields = False
    if file_path.exists():
        try:
            json = loads(file_path.read_text("utf-8"))
            check_fields = True
        except:
            json = defaults
    else:
        json = defaults
    
    if check_fields:
        for name in defaults:
            if name not in json:
                json[name] = defaults[name]
    
    botm.config.set_config(file_path, json)""")
    
    mkmodule(cwd / "main.py", """\
from telegram.ext import CommandHandler
from .app import botm, application
from .screens import load_screens
from .config import load_config
from .start import start, start_inner

load_screens()
load_config()

botm.start_inner = start_inner

# application.job_queue.run_repeating(action, interval=60, first=1)

application.add_handler(CommandHandler("start", start), 0)
botm.add_handlers()

print("Запрашивание...")
application.run_polling(0.1)""")

    mkmodule(cwd / "screens.py", R"""\
import importlib
from pathlib import Path

def load_screens():
    p = Path("src/screen")
    for path in p.rglob("*.py"):
        if path.name.startswith("_"):
            continue
        fullpath = str(path)
        module_name = fullpath[:-3].replace("/", ".").replace("\\", ".")
        importlib.import_module(module_name)""")
    
    mkmodule(cwd / "start.py", """\
from telegram import Update
from src.init.app import botm

async def start(update: Update, _):
    user_id = update.message.from_user.id
    print(f"{user_id} написал")
    await start_inner(user_id) 

async def start_inner(user_id: int):    
    sud = botm.system_user_data.get(user_id)
    if sud and sud.screen:
        try: await sud.screen.delete(botm.bot)
        except: ...
    
    botm.system_user_data.reset(user_id)
    botm.reset_user_data(user_id)
    
    await botm.screen.set_by_name(user_id, "welcome")""")


from .common import *

def create_model(cwd: Path): 
    mkpackage(cwd)
    mkmodule(cwd / "bot_manager.py", """\
from typing import Callable
from telegram.ext import Application
from tg_bot_screen.ptb import BotManager as BaseBotManager
from .user_data import UserDataManager
from .config_manager import ConfigManager

class BotManager(BaseBotManager):
    def __init__(self, application: Application):
        super().__init__(application)
        self.user_data_m = UserDataManager()
        self.config = ConfigManager()
        
        self.start_inner: Callable = None
    
    def get_user_data(self, user_id: int):
        return self.user_data_m.get(user_id)

    def reset_user_data(self, user_id: int):
        self.user_data_m.reset(user_id)
        
    async def mapping_key_error(self, user_id: int):
        await self.start_inner(user_id)""")
    
    mkmodule(cwd / "config_manager.py", """\
from json import dumps
from pathlib import Path
from typing import Any

class ConfigManager:
    def __init__(self):
        self.__path: Path = None
        self.__json: dict[str, Any] = None
        self.defaults = {
            "test" : 0
        }
    
    def set_config(self, path: Path, json: dict):
        self.__path = path
        self.__json = json
        self.dump_to_file()
    
    def dump_to_file(self):
        self.__path.touch(exist_ok=True)
        self.__path.write_text(dumps(self.__json, indent=4, ensure_ascii=False), 
            encoding="utf-8")
    
    @property
    def test(self) -> int:
        return self.__json["test"]
    
    @test.setter
    def test(self, value: int):
        self.__json["test"] = value
        self.dump_to_file()
    """)
    mkmodule(cwd / "user_data.py", """\
class UserData:
    def __init__(self):
        self.last_error: str = None 

class UserDataManager:
    def __init__(self):
        self.users_data: dict[int, UserData] = {}
    
    def get(self, user_id: int):
        if user_id not in self.users_data:
            ud = UserData()
            self.users_data[user_id] = ud
            return ud
        
        return self.users_data[user_id]

    def reset(self, user_id: int):
        self.users_data[user_id] = UserData()""")